package main

import (
	"bytes"
	"errors"
	"fmt"
	"html/template"
	"io"
	"log"
	"net/http"
	"strconv"
	"strings"
)

var detailsTemplate = template.Must(template.ParseFS(templateFS, "templates/base.tpl", "templates/details.tpl"))

func (h Handler) img(w http.ResponseWriter, r *http.Request) {
	logRequest(r)

	if r.Method != "GET" {
		httpError(w, "bad method", errors.New(r.Method), http.StatusMethodNotAllowed)
		return
	}

	args := r.URL.Query()
	log.Println("args", args)

	if args.Get("id") == "" {
		httpError(w, "bad request", errors.New("empty id"), http.StatusBadRequest)
		return
	}

	id, err := strconv.Atoi(args.Get("id"))
	if err != nil {
		httpError(w, "can't parse id", err, http.StatusBadRequest)
		return
	}

	img, err := LoadPictureData(r.Context(), h.db, id)
	if err != nil {
		// TODO: use more appropriate status code
		httpError(w, "can't load image", err, http.StatusNotFound)
		return
	}

	w.Header().Set("Content-Type", "image/png")

	n, err := io.Copy(w, bytes.NewBuffer(img))
	if err != nil {
		log.Println("can't write image:", err)
		return
	}

	log.Printf("wrote image for %d: %d bytes", id, n)
}

func (h Handler) detailsPost(w http.ResponseWriter, r *http.Request) {
	log.Println("handling", r.Method, r.URL, "from", r.RemoteAddr)

	action := r.FormValue("action")
	switch action {
	case "save", "delete":
	default:
		httpError(w, "invalid parameters", fmt.Errorf("action missing or unknown: %q", action), http.StatusBadRequest)
		return
	}

	id, err := strconv.Atoi(r.FormValue("id"))
	if err != nil {
		httpError(w, "can't parse id", err, http.StatusBadRequest)
		return
	}

	if action == "delete" {
		err = DeleteVino(r.Context(), h.db, id)
		if err != nil {
			httpError(w, "can't delete wine", err, http.StatusInternalServerError)
			return
		}

		http.Redirect(w, r, "/", http.StatusSeeOther)
		return
	}

	name := r.FormValue("name")
	if name == "" {
		httpError(w, "name can't be empty", nil, http.StatusBadRequest)
		return
	}

	rating, err := strconv.Atoi(r.FormValue("rating"))
	if err != nil {
		httpError(w, "can't parse rating", nil, http.StatusBadRequest)
		return
	}

	v, err := LoadVino(r.Context(), h.db, id)
	if err != nil {
		// Only log error here, just create a new entry.
		log.Println("can't load vino with id", id, ":", err)
	}

	v.Name = name
	v.Rating = rating

	comment := strings.TrimSpace(r.FormValue("comment"))
	if comment != "" {
		if len(comment) > 1024 {
			comment = comment[:1024] + "..."
		}

		err = v.StoreComment(r.Context(), h.db, comment)
		if err != nil {
			httpError(w, "can't add comment", err, http.StatusInternalServerError)
			return
		}
	}

	err = r.ParseMultipartForm(16 * 1024 * 1024)
	if err != nil {
		httpError(w, "can't parse multipart form data", err, http.StatusInternalServerError)
		return
	}

	// See if there's an image file uploaded
	if len(r.MultipartForm.File["picture"]) != 0 {
		picFile, picHdr, err := r.FormFile("picture")
		if err != nil {
			httpError(w, "can't open picture file", err, http.StatusInternalServerError)
			return
		}
		defer picFile.Close()

		err = v.AddPicture(picFile, picHdr.Header.Get("Content-Type"))
		if err != nil {
			httpError(w, "can't load picture", err, http.StatusBadRequest)
			return
		}
	}

	err = v.Store(r.Context(), h.db, Update)
	if err != nil {
		httpError(w, "can't store wine", err, http.StatusInternalServerError)
		return
	}

	http.Redirect(w, r, "/details?id="+r.FormValue("id"), http.StatusSeeOther)
	return
}

func (h Handler) details(w http.ResponseWriter, r *http.Request) {
	if r.Method == "POST" {
		h.detailsPost(w, r)
		return
	}

	log.Println("handling", r.Method, r.URL, "from", r.RemoteAddr)

	if r.Method != "GET" {
		httpError(w, "bad method", errors.New(r.Method), http.StatusMethodNotAllowed)
		return
	}

	args := r.URL.Query()
	log.Println("args", args)

	if args.Get("id") == "" {
		httpError(w, "bad request", errors.New("empty id"), http.StatusBadRequest)
		return
	}

	id, err := strconv.Atoi(args.Get("id"))
	if err != nil {
		httpError(w, "can't parse id", err, http.StatusBadRequest)
		return
	}

	v, err := LoadVino(r.Context(), h.db, id)
	if err != nil {
		httpError(w, "can't load wine data", err, http.StatusInternalServerError)
		return
	}

	data := struct {
		Wine Vino
	}{v}

	err = detailsTemplate.ExecuteTemplate(w, "details.tpl", data)
	if err != nil {
		log.Println("can't execute details template:", err)
		return
	}
}
